/*****************************************************************
JADE - Java Agent DEvelopment Framework is a framework to develop 
multi-agent systems in compliance with the FIPA specifications.
Copyright (C) 2000 CSELT S.p.A. 

GNU Lesser General Public License

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, 
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
*****************************************************************/



package AgentiMobili;

import java.io.IOException;
import java.util.LinkedList;
import java.util.StringTokenizer;

import alice.tuprolog.MalformedGoalException;
import alice.tuprolog.NoMoreSolutionException;
import alice.tuprolog.NoSolutionException;
import alice.tuprolog.Prolog;
import alice.tuprolog.SolveInfo;
import alice.tuprolog.UnknownVarException;

import Ontologia.Action;
import Ontologia.Explorable;
import Ontologia.Tile;
import Ontologia.TileOntology;

import jade.core.*;
import jade.core.behaviours.*;

import jade.domain.mobility.*;

import jade.lang.acl.*;

/**
This behaviour of the Agent serves all the received messages. In particular,
the following expressions are accepted as content of "request" messages:
- (move <destination>)  to move the Agent to another container. Example (move Front-End) or
(move (:location (:name Container-1) (:transport-protocol JADE-IPMT) (:transport-address IOR:0000...) ))
- (exit) to request the agent to exit
- (stop) to stop the counter
- (continue) to continue counting
@author Giovanni Caire - CSELT S.p.A
@version $Date: 2002/07/18 00:22:29 $ $Revision: 2.4 $
*/
class ServeIncomingMessagesBehaviour extends SimpleBehaviour
{
	ServeIncomingMessagesBehaviour(Agent a)
	{
		super(a);
	}

	public boolean done()
	{
		return false;
	}

	@SuppressWarnings("unchecked")
	public void action()
	{
		ACLMessage msg, msg2,msg3;
		MessageTemplate mt = MessageTemplate.MatchPerformative(ACLMessage.REQUEST);
        MessageTemplate mtOnto = MessageTemplate.MatchOntology(TileOntology.getInstance().getName());
		MessageTemplate mtSender = MessageTemplate.MatchSender(((MobileAgent)myAgent).getActualWorldId());
        MessageTemplate mt2= MessageTemplate.and(mtOnto,mtSender);
        MessageTemplate mt3 = MessageTemplate.MatchContent("AggiungiRisorsa");
        
        // Get a message from the queue or wait for a new one if queue is empty
		msg = myAgent.receive(mt);
	    msg2 = myAgent.receive(mt2);
		msg3 =myAgent.receive(mt3);
	    if (msg == null && msg2 ==null && msg3==null) {
			block();
		 	return;
		}
		else {
			if(msg3!=null){
				
				((MobileAgent)myAgent).treeCounter++;
				
				
			}
			
			
			
			
			if(msg!=null){
			String replySentence = new String("");

			// Get action to perform
			//String s = msg.getContent().
			StringTokenizer st = new StringTokenizer(msg.getContent(), " ()\t\n\r\f");
			String action = (st.nextToken()).toLowerCase();
			// EXIT
			if      (action.equals("exit"))
			{
				System.out.println("They requested me to exit (Sob!)");
				// Set reply sentence
				replySentence = new String("\"OK exiting\"");
				myAgent.doDelete();
			}
			// STOP COUNTING
			else if (action.equals("stop"))
			{
				System.out.println("They requested me to stop counting");
				((MobileAgent) myAgent).stopCounter();
				// Set reply sentence
				replySentence = new String("\"OK stopping\"");
			} 				
			// CONTINUE COUNTING
			else if (action.equals("continue"))
			{
				System.out.println("They requested me to continue counting");
				((MobileAgent) myAgent).continueCounter();
				// Set reply sentence
				replySentence = new String("\"OK continuing\"");
			} 
			// MOVE TO ANOTHER LOCATION				
			else if (action.equals("move"))
			{
			    String destination = st.nextToken();
			    System.out.println();
			    Location dest = new jade.core.ContainerID(destination, null);
			    System.out.println("They requested me to go to " + destination);
				// Set reply sentence
				replySentence = new String("\"OK moving to " + destination+" \"");
				// Prepare to move
				((MobileAgent)myAgent).nextSite = dest;
				myAgent.doMove(dest);
			}
			// CLONE TO ANOTHER LOCATION				
			else if (action.equals("clone"))
			{
			    String destination = st.nextToken();
			    System.out.println();
			    Location dest = new jade.core.ContainerID(destination, null);
			    System.out.println("They requested me to clone myself to " + destination);
				// Set reply sentence
				replySentence = new String("\"OK cloning to " + destination+" \"");
				// Prepare to move
				((MobileAgent)myAgent).nextSite = dest;
				myAgent.doClone(dest, "clone"+((MobileAgent)myAgent).cnt+"of"+myAgent.getName());
			}
			// SAY THE CURRENT LOCATION 
			else if (action.equals("where-are-you"))
			{
			    System.out.println();
			    Location current = myAgent.here();
			    System.out.println("Currently I am running on "+current.getName());
				// Set reply sentence
				replySentence = current.getName();
			}

			// Reply
			ACLMessage replyMsg = msg.createReply();
			replyMsg.setPerformative(ACLMessage.INFORM);
			replyMsg.setContent(replySentence);
			myAgent.send(replyMsg);
		}
			
		if(msg2!=null){
			
			Object ce=null;
			try {
				
				ce =  msg2.getContentObject();
			} catch (UnreadableException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			if(ce instanceof Explorable){
				//controllare la matrice alla posizione tot, se
				//  a posto manda un oggetto Explorable spiegando
				// al suo interno che cosa c' nella casella.	
					//System.out.println("Ci sono Tile"+ ((Tile)ce).getTileX()+ " "+((Tile)ce).getTileY());
					Explorable t= (Explorable)ce;
				
				try {
					//non ci devono essere duplicati della casella nella teoria logica
					
			        
					//System.out.println(((MobileAgent)myAgent).getEngine().getTheory().toString());
					
					//metto la tupla nuova
					((MobileAgent)myAgent).getEngine().solve("in(tile("+t.getX()+","+ t.getY() +",_)).");
					((MobileAgent)myAgent).getEngine().solve("out(tile("+t.getX()+","+ t.getY() +","+t.getType()+")).");
					if(t.getStatus()==true){
						if(t.getType().equalsIgnoreCase("free")){
						ACLMessage msgs = new ACLMessage (ACLMessage.UNKNOWN);
						msgs.addReceiver(msg2.getSender());
						msgs.setOntology(msg2.getOntology());
						msgs.setContentObject(new Action("GoTo",t.getX(),t.getY()));
						myAgent.send(msgs);
						Tile nTile= new Tile(t.getX(),t.getY());
						 ((MobileAgent)myAgent).setTile(nTile);}
						else if(t.getType().equalsIgnoreCase("Tree")){
							ACLMessage msgs = new ACLMessage (ACLMessage.UNKNOWN);
							msgs.addReceiver(msg2.getSender());
							msgs.setOntology(msg2.getOntology());
							msgs.setContentObject(new Action("Take",t.getX(),t.getY()));
							myAgent.send(msgs);
							Tile nTile= new Tile(t.getX(),t.getY());
							 ((MobileAgent)myAgent).setTile(nTile);
							 ((MobileAgent)myAgent).treeCounter++;
							 ((MobileAgent)myAgent).getEngine().solve("in(tile("+t.getX()+","+ t.getY() +",_)).");
						     ((MobileAgent)myAgent).getEngine().solve("out(tile("+t.getX()+","+ t.getY() +",free)).");
						}
						     
						     
						     System.out.println(((MobileAgent)myAgent).getEngine().solve("risorse("+((MobileAgent)myAgent).treeCounter+").").isSuccess());
						    
						   
						   
						   if(((MobileAgent)myAgent).getEngine().solve("risorse("+((MobileAgent)myAgent).treeCounter+").").isSuccess()
							 && ((MobileAgent)myAgent).cnt%10==0)
						   {
							   
							   
							   
							   @SuppressWarnings("unused") LinkedList<Tile> SoluzioniFree=new LinkedList();
							   
						    	 try{
						    	SolveInfo info= ((MobileAgent)myAgent).getEngine().solve("rd(tile(X,Y,free)).");
						    	Prolog engine =((MobileAgent)myAgent).getEngine();
						    	while (info.isSuccess()){
						    		 System.out.println(
						    		 "solution: "+info.getSolution()+
						    		 " - bindings: X/"+info.getTerm("X")+
						    		 " Y/"+info.getTerm("Y"));
						    		 SoluzioniFree.add(new Tile(Integer.parseInt(info.getTerm("X").toString()),Integer.parseInt(info.getTerm("Y").toString())));
						    	if (engine.hasOpenAlternatives()){
						    		 info=engine.solveNext();
						    		 } else {
						    		 break;
						    		 } }
						    	
						    	for(int counter =0; counter<SoluzioniFree.size();counter++){
						    	SolveInfo info2 = ((MobileAgent)myAgent).getEngine().solve("c(tile("+SoluzioniFree.get(counter).getTileX()+","+ SoluzioniFree.get(counter).getTileY()+",free),_,_,_,risorse("+((MobileAgent)myAgent).treeCounter+")).");
					    		 
						    	if(info2.isSuccess()){
						    		System.out.println("successo"+info2.getSolution().toString());
						    		((MobileAgent)myAgent).building=true;
						    		((MobileAgent)myAgent).BuildingTile=SoluzioniFree.get(counter);
						    		
						    		SoluzioniFree.clear();
						    		break;
						    	}	 
						    	}
						    	SoluzioniFree.clear();
						    	
						    	 }catch (NoSolutionException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								} catch (UnknownVarException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								} catch (NoMoreSolutionException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
						     
//								
						     
						     }
						
						
						
						
						
					}
					
					
					
				} catch (MalformedGoalException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			
			
			}
			
			
			
			
			
		}	
			
			
			
			
			
		}

		
		
		
		
		return;
	}
}

